// This sample code tests the performance of sorting provided by
// "szsort.h". Strings in files specified by command line arguments are
// sorted by StringSort::Sort() and std::sort().
#include "szsort.h"

#include <cassert>
#include <ctime>

#include <fstream>
#include <string>
#include <vector>

using namespace std;


namespace {

// Timer.
void Timer(const char *message) {
  static clock_t cl;
  if (message == NULL)
    cl = clock();
  else
    printf("%s%.3f sec\n", message, 1.0 * (clock() - cl) / CLOCKS_PER_SEC);
}

// Function object to get a letter from a key.
struct GetLetter {
  unsigned char operator()(string *key, size_t depth) {
    return (unsigned char)(*key)[depth];
  }
};

// Function object to compare strings.
struct CompareStrings {
  bool operator()(string *lhs, string *rhs) {
    return *lhs < *rhs;
  }
};

// Confirm that an array is sorted.
template <typename RandomAccessIterator>
void IsSorted(RandomAccessIterator l, RandomAccessIterator r) {
  while (++l < r)
    assert(**(l - 1) <= **l);
}

// Test using files.
void TestForFiles(char **files, int num_files) {
  vector<string *> keys;
  for (int i = 0; i < num_files; ++i) {
    ifstream input(files[i]);
    string line;
    while (getline(input, line))
      keys.push_back(new string(line));
  }

  // Multikey quicksort.
  vector<string *> tmp(keys);
  Timer(NULL);
  StringSort::Sort(tmp.begin(), tmp.end(), 0, GetLetter());
  Timer("StringSort::Sort(): ");
  IsSorted(tmp.begin(), tmp.end());

  // Standard sorting algorithm.
  Timer(NULL);
  sort(keys.begin(), keys.end(), CompareStrings());
  Timer("std::sort(): ");
  IsSorted(keys.begin(), keys.end());

  for (vector<string *>::iterator it = keys.begin(); it != keys.end(); ++it)
    delete *it;
}

}  // namespace.


int main(int argc, char *argv[]) {
  assert(argc > 1);
  TestForFiles(argv + 1, argc - 1);
  return 0;
}
